#include "light.h"
#include <util/delay.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
//lower and upper margin of futaba s3003
//experimental results
#define OFFSET 700
#define MAX180 2300 
int photo[8];
int phases[]={0,60,120,180,240,300,60,120};
//some corrections
int phases2[]={-10,0,0,180,0,0,30,130};
int current1;
/************************************************************************ 
 * REDIRECT stdin, stdout for serial transmition
 ************************************************************************/
int uart_putchar(char c, FILE *unused);
int uart_getchar(FILE* f);
FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);
FILE mystdin =  FDEV_SETUP_STREAM(NULL, uart_getchar, _FDEV_SETUP_READ);

void USART_init(void)
{
	// Initializare modul USART pt comunicatie la 9600bps, 8 biti de date, la 16MHz											
	UBRRH = 0;				
	UBRRL = 103;
	UCSRB = 0x18;
	UCSRC = _BV(URSEL) | _BV(UCSZ1) | _BV(UCSZ0);
}

int uart_putchar(char c, FILE *unused)      
{
	if (c == '\n')
		uart_putchar('\r', 0);
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = c;
	return 0;
}

int uart_getchar(FILE* f)
{
	char c;
    
    loop_until_bit_is_set(UCSRA, RXC);
    c = UDR;
    if(c=='\r')
		c='\n';
    uart_putchar(c,NULL); 
    return c;
}



void init_timer1(){
	//top=20000, prescaler=8, frec=50hz (T=20ms)
	ICR1=20000;
	TCCR1B|=_BV(CS11);
	
	//phase and frequency correction, icr1 = top
	TCCR1B|=_BV(WGM13);
	//set OC1A,OC2B on downcounting, clear on upcounting
	TCCR1A|=_BV(COM1A1)|_BV(COM1B1);
	//output pins, PORTD4 (motor 2) and PORTD5 (motor 1)
	DDRD|=_BV(4)|_BV(5);
	
	//default position
	OCR1A=OFFSET;//portd5
	OCR1B=OFFSET;//portd4
	
}


int get_pulse1(int degrees){
//get the pulse for the bottom motor
	if(degrees>180)
		degrees-=180;	
	return get_pulse2(degrees);	
}

void set_pulse1(int degrees){
	OCR1A=get_pulse1(degrees);
}

int get_pulse2(int degrees){
	return (int)((float)degrees*(MAX180-OFFSET)/180+OFFSET);
}

void set_pulse2(int degrees){
	OCR1B=get_pulse2(degrees);
}

int ADC_get(uint8_t channel)
{

	ADMUX &= 0xE0;
	
	ADMUX |= channel;
	ADCSRA |= _BV(ADEN)|_BV(ADSC);
	while(!(ADCSRA & (1<<ADIF)));
	return ADC;
}

void ADC_init(void)
{
	//PORTA - all inputs
	DDRA=0;
	// AVCC with capacitor on AREF;
	// result left-adjusted (8 bit data)
	ADMUX = (1<<REFS0);
	// max prescaler
	ADCSRA = (1<<ADEN) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);

}



int main(void){
	int k,min;
	
	//for serial transmition
	stdin = &mystdin;
	stdout = &mystdout;
	
	//All A-ports are inputs
	DDRA=0;
	ADC_init();
	USART_init();
	init_timer1();
	current1=0;

	_delay_ms(2000);
	while(1){

		min=0;
		//get all ADC values needed by motor 1
		for(k=0;k<6;k++)
			photo[k]=ADC_get(k);
		printf("ADCs are: \n%d %d %d %d\n%d %d \n",photo[0],photo[1],
		photo[2],photo[3],photo[4],photo[5]);
		
		
		for(k=0;k<6;k++)
			if(photo[min]>photo[k])
				min=k;
		printf("Min is %d[%d angle] (%d)\n",min,phases[min],photo[min]);	
	
		current1+=phases[min];
		while(current1>180)
			current1-=180;
		set_pulse1(current1);
		_delay_ms(2000);
			
			
			
		min=0;
		//get or get again all ADC values needed by motor 2
		for(k=6;k<8;k++)
			photo[k]=ADC_get(k);
		photo[0]=ADC_get(0);
		photo[3]=ADC_get(3);
		
		for(k=6;k<8;k++)
			if(photo[min]>photo[k])
				min=k;
		if(photo[min]>photo[0])
			min=0;
		if(photo[min]>photo[3])
			min=3;
		printf("Min2 for Servo2 is %d[%d angle] (%d)\n",min,phases2[min],photo[min]);	
		set_pulse2(phases2[min]);
		
		//refresh each 4 seconds
		_delay_ms(4000);
			

		
	}
	return 0;
}
